'use strict';

var Ui = require('kenga/utils');
var Label = require('kenga-labels/label');
var HolyGrailPane = require('kenga-containers/holy-grail-pane');
var Box = require('kenga-containers/box-pane');
var SplitPane = require('kenga-containers/split-pane');
var FlowPane = require('kenga-containers/flow-pane');
var Toolbar = require('kenga-containers/tool-bar');
var ScrollPane = require('kenga-containers/scroll-pane');
var Button = require('kenga-buttons/button');
var DataGrid = require('kenga-grid/grid');
var ColumnNode = require('kenga-grid/columns/column-node');
var Menu = require('kenga-menu/menu');
var MenuItem = require('kenga-menu/menu-item');
var MenuSeparator = require('kenga-menu/menu-separator');
var i18n = require('./i18n.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Ui__default = /*#__PURE__*/_interopDefaultLegacy(Ui);
var Label__default = /*#__PURE__*/_interopDefaultLegacy(Label);
var HolyGrailPane__default = /*#__PURE__*/_interopDefaultLegacy(HolyGrailPane);
var Box__default = /*#__PURE__*/_interopDefaultLegacy(Box);
var SplitPane__default = /*#__PURE__*/_interopDefaultLegacy(SplitPane);
var FlowPane__default = /*#__PURE__*/_interopDefaultLegacy(FlowPane);
var Toolbar__default = /*#__PURE__*/_interopDefaultLegacy(Toolbar);
var ScrollPane__default = /*#__PURE__*/_interopDefaultLegacy(ScrollPane);
var Button__default = /*#__PURE__*/_interopDefaultLegacy(Button);
var DataGrid__default = /*#__PURE__*/_interopDefaultLegacy(DataGrid);
var ColumnNode__default = /*#__PURE__*/_interopDefaultLegacy(ColumnNode);
var Menu__default = /*#__PURE__*/_interopDefaultLegacy(Menu);
var MenuItem__default = /*#__PURE__*/_interopDefaultLegacy(MenuItem);
var MenuSeparator__default = /*#__PURE__*/_interopDefaultLegacy(MenuSeparator);

function Layout() {
  var ground = new HolyGrailPane__default['default']();
  {
    ground.focusable = true;
  }
  var tools = new Toolbar__default['default']();
  var palette = new Box__default['default'](Ui__default['default'].Orientation.VERTICAL);
  var explorer = new DataGrid__default['default']();
  var widgetColumn = new ColumnNode__default['default']();
  var leftBox = new Box__default['default']();
  var leftSizer = new FlowPane__default['default']();
  var rightBox = new Box__default['default']();
  var rightSizer = new FlowPane__default['default']();
  var paletteExplorerSplit = new SplitPane__default['default']();
  var explorerMenu = new Menu__default['default']();
  var propertiesBox = new HolyGrailPane__default['default']();
  var propertiesHeader = new Label__default['default'](i18n['winnie.properties.header']);
  var properties = new DataGrid__default['default']();
  var widgets = new FlowPane__default['default']();
  widgets.vgap = null;
  widgets.hgap = null;
  {
    widgets.focusable = true;
    widgetColumn.field = 'name';
    widgetColumn.title = 'widget.name';
    widgetColumn.width = null;
    explorer.addColumnNode(widgetColumn);
    explorer.headerVisible = false;
    explorer.insertable = explorer.deletable = false;
    explorer.draggableRows = true;
    explorer.showHorizontalLines = explorer.showVerticalLines = explorer.showOddRowsInOtherColor = false;
    explorer.contextMenu = explorerMenu;
    paletteExplorerSplit.dividerSize /= 2;
    paletteExplorerSplit.width = 340;
    paletteExplorerSplit.orientation = Ui__default['default'].Orientation.VERTICAL;
    paletteExplorerSplit.first = palette;
    paletteExplorerSplit.second = explorer;
  }
  var propNameColumn = new ColumnNode__default['default']();
  var propValueColumn = new ColumnNode__default['default']();
  {
    propertiesBox.width = 310;
    propNameColumn.title = i18n['winnie.prop.name'];
    propNameColumn.field = 'name';
    propNameColumn.readonly = true;
    propNameColumn.width = null;
    propValueColumn.title = i18n['winnie.prop.value'];
    propValueColumn.field = 'value';
    propValueColumn.width = null;
    properties.addColumnNode(propNameColumn);
    properties.addColumnNode(propValueColumn);
    properties.insertable = properties.deletable = false;
    properties.headerVisible = false;
    propertiesBox.header = propertiesHeader;
    propertiesBox.content = properties;
  }
  var view = new ScrollPane__default['default'](widgets);
  {
    view.contextMenu = explorerMenu;
    leftBox.add(paletteExplorerSplit);
    leftBox.add(leftSizer);
    rightBox.add(rightSizer);
    rightBox.add(propertiesBox);
    ground.header = tools;
    ground.leftSide = leftBox;
    ground.content = view;
    ground.rightSide = rightBox;
  }
  var tTemplates = new Button__default['default']();
  var templatesMenu = new Menu__default['default']();
  var tOpen = new Button__default['default']();
  var tSave = new Button__default['default']();
  var tExport = new Button__default['default']();
  var tCut = new Button__default['default']();
  var tCopy = new Button__default['default']();
  var tPaste = new Button__default['default']();
  var tRemove = new Button__default['default']();
  var tUndo = new Button__default['default']();
  var tRedo = new Button__default['default']();
  var tSettings = new Button__default['default']();

  function withStyle(name) {
    var div = document.createElement('div');
    div.className = name;
    return div;
  }

  {
    tTemplates.icon = withStyle('winnie-icon-buffer');
    tTemplates.toolTipText = i18n['winnie.adopt.tooltip'];
    tOpen.icon = withStyle('winnie-icon-folder-open-empty');
    tOpen.toolTipText = i18n['winnie.open.tooltip'];
    tSave.icon = withStyle('winnie-icon-floppy');
    tSave.toolTipText = i18n['winnie.save.tooltip'];
    tExport.icon = withStyle('winnie-icon-export');
    tExport.toolTipText = i18n['winnie.export.tooltip'];
    tCut.icon = withStyle('winnie-icon-scissors');
    tCut.toolTipText = i18n['winnie.cut.tooltip'];
    tCopy.icon = withStyle('winnie-icon-docs');
    tCopy.toolTipText = i18n['winnie.copy.tooltip'];
    tPaste.icon = withStyle('winnie-icon-clipboard');
    tPaste.toolTipText = i18n['winnie.paste.tooltip'];
    tRemove.icon = withStyle('winnie-icon-trash');
    tRemove.toolTipText = i18n['winnie.remove.tooltip'];
    tSettings.icon = withStyle('winnie-icon-cogs');
    tSettings.toolTipText = i18n['winnie.settings.tooltip'];
    tUndo.icon = withStyle('winnie-icon-reply-1');
    tUndo.toolTipText = i18n['winnie.undo.tooltip'];
    tRedo.icon = withStyle('winnie-icon-forward-1');
    tRedo.toolTipText = i18n['winnie.redo.tooltip'];
    tools.add(tTemplates);
    tools.add(tOpen);
    tools.add(tSave);
    tools.add(tExport);
    tools.add(tCut);
    tools.add(tCopy);
    tools.add(tPaste);
    tools.add(tUndo);
    tools.add(tRedo);
    tools.add(tRemove); //tools.add(tSettings);
  }
  var miAdd = new MenuItem__default['default'](i18n['winnie.add.name']);
  miAdd.subMenu = new Menu__default['default']();
  var miToSurface = new MenuItem__default['default'](i18n['winnie.to.surface.name']);
  var miCut = new MenuItem__default['default'](i18n['winnie.cut.name']);
  var miCopy = new MenuItem__default['default'](i18n['winnie.copy.name']);
  var miPaste = new MenuItem__default['default'](i18n['winnie.paste.name']);
  var miUndo = new MenuItem__default['default'](i18n['winnie.undo.name']);
  var miRedo = new MenuItem__default['default'](i18n['winnie.redo.name']);
  var miRemove = new MenuItem__default['default'](i18n['winnie.remove.name']);
  {
    miAdd.icon = withStyle('winnie-icon-space');
    miToSurface.icon = withStyle('winnie-icon-space');
    miCut.icon = withStyle('winnie-icon-scissors');
    miCopy.icon = withStyle('winnie-icon-docs');
    miPaste.icon = withStyle('winnie-icon-clipboard');
    miUndo.icon = withStyle('winnie-icon-reply-1');
    miRedo.icon = withStyle('winnie-icon-forward-1');
    miRemove.icon = withStyle('winnie-icon-trash');
    [miAdd, miToSurface, miCut, miCopy, miPaste, miUndo, miRedo, miRemove].forEach(function (mi) {
      mi.iconTextGap = null;
    });
    explorerMenu.add(miAdd);
    explorerMenu.add(new MenuSeparator__default['default']());
    explorerMenu.add(miToSurface);
    explorerMenu.add(new MenuSeparator__default['default']());
    explorerMenu.add(miCut);
    explorerMenu.add(miCopy); //explorerMenu.add(miPaste);

    explorerMenu.add(new MenuSeparator__default['default']());
    explorerMenu.add(miUndo);
    explorerMenu.add(miRedo);
    explorerMenu.add(new MenuSeparator__default['default']());
    explorerMenu.add(miRemove);
  }
  return {
    ground: ground,
    tools: tools,
    palette: palette,
    widgetColumn: widgetColumn,
    explorer: explorer,
    paletteExplorerSplit: paletteExplorerSplit,
    leftBox: leftBox,
    leftSizer: leftSizer,
    explorerMenu: explorerMenu,
    propertiesBox: propertiesBox,
    propertiesHeader: propertiesHeader,
    properties: properties,
    rightBox: rightBox,
    rightSizer: rightSizer,
    widgets: widgets,
    view: view,
    miAdd: miAdd,
    miToSurface: miToSurface,
    miCut: miCut,
    miCopy: miCopy,
    miPaste: miPaste,
    miUndo: miUndo,
    miRedo: miRedo,
    miRemove: miRemove,
    propNameColumn: propNameColumn,
    propValueColumn: propValueColumn,
    tOpen: tOpen,
    tTemplates: tTemplates,
    templatesMenu: templatesMenu,
    tSave: tSave,
    tExport: tExport,
    tCut: tCut,
    tCopy: tCopy,
    tPaste: tPaste,
    tUndo: tUndo,
    tRedo: tRedo,
    tRemove: tRemove,
    tSettings: tSettings
  };
}

module.exports = Layout;
