'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var Ui = require('kenga/utils');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Ui__default = /*#__PURE__*/_interopDefaultLegacy(Ui);

function startRectSelection(rectHost, event) {
  var viewX = Ui__default['default'].absoluteLeft(rectHost);
  var viewY = Ui__default['default'].absoluteTop(rectHost);
  var rect = document.createElement('div');
  rect.className = 'p-winnie-selection-rect';
  rectHost.appendChild(rect);
  rect.style.position = 'absolute';
  var x = event.clientX - viewX + rectHost.scrollLeft;
  var y = event.clientY - viewY + rectHost.scrollTop;
  rect.style.left = "".concat(x, "px");
  rect.style.top = "".concat(y, "px");
  rect.style.width = rect.style.height = '0px';
  return {
    x: x,
    y: y,
    rect: rect
  };
}

function proceedRectSelection(start, diff) {
  if (diff.x > 0) {
    start.rect.style.left = "".concat(start.x, "px");
    start.rect.style.width = "".concat(diff.x, "px");
  } else {
    start.rect.style.left = "".concat(start.x + diff.x, "px");
    start.rect.style.width = "".concat(-diff.x, "px");
  }

  if (diff.y > 0) {
    start.rect.style.top = "".concat(start.y, "px");
    start.rect.style.height = "".concat(diff.y, "px");
  } else {
    start.rect.style.top = "".concat(start.y + diff.y, "px");
    start.rect.style.height = "".concat(-diff.y, "px");
  }
}

function endRectSelection(model, start, diff, event) {
  function hitTestElement(rectX, rectY, element) {
    if (element['p-widget'] && element['p-widget']['winnie.wrapper']) {
      var wX = Ui__default['default'].absoluteLeft(element);
      var wY = Ui__default['default'].absoluteTop(element);

      if (rectX < wX + element.offsetWidth && wX < rectX + start.rect.offsetWidth && rectY < wY + element.offsetHeight && wY < rectY + start.rect.offsetHeight) {
        model.layout.explorer.select(element['p-widget']['winnie.wrapper']);
        return true;
      }
    }

    return false;
  }

  var visualRootElement = model.visualRootElement();

  if (visualRootElement) {
    if (!event.ctrlKey) {
      model.layout.explorer.unselectAll();
    }

    var rectX = Ui__default['default'].absoluteLeft(start.rect);
    var rectY = Ui__default['default'].absoluteTop(start.rect);
    var child = visualRootElement.firstElementChild;
    var hitted = 0;

    while (child) {
      if (hitTestElement(rectX, rectY, child)) {
        hitted++;
      }

      if (hitted > 0 && diff.x === 0 && diff.y === 0) {
        break;
      }

      child = child.nextElementSibling;
    }

    if (hitted === 0 && diff.x === 0 && diff.y === 0) {
      hitTestElement(rectX, rectY, visualRootElement);
    }
  }

  start.rect.parentElement.removeChild(start.rect);
}

exports.endRectSelection = endRectSelection;
exports.proceedRectSelection = proceedRectSelection;
exports.startRectSelection = startRectSelection;
